﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;

//This page is a template for ACH PaymentGateway.net Secure Web Pay (SWP) transactions
//The code builds an HTML string with the appropriate field-value pairs and then posts that to SWP
//Set your SWP access values in section "Enter your values here" (lines 53-57)
//Additional controls can be added to the page
//and then those values passed to SWP by adding the field-value pairs in "Optional Page Values" (line 84)

//Code is provided as-is with no warranty
//Clint A. Andera 2009-02-26

public partial class Post : System.Web.UI.Page
{
    protected void Page_Load(object sender, EventArgs e)
    {
        //To clear the Cache-- don't want to post the same transaction 2x if user tries to back up
        Response.Cache.SetExpires(DateTime.Now);
        Response.Cache.SetCacheability(HttpCacheability.NoCache);
        
        if (!Page.IsPostBack)
        {
            //Set utcTime, store on page in hidden field
            page_pg_utc_time.Value = DateTime.UtcNow.Ticks.ToString();

            //Set SWP Transaction Order Number, store on page in hidden field
                //SWP defines this as 'Random number identifying the transaction'
                //I don't know if this number should be stored in the local DB for reference or not...
                //I just pass pg_consumerorderid with my number to match against my DB records
            int RandomNumber;
            Random RandomClass = new Random();
            RandomNumber = RandomClass.Next(1000, 1000000);

            page_pg_transaction_order_number.Value = RandomNumber.ToString();

        }
    }

    protected void btnSWP_Click(object sender, EventArgs e)
    {
        string apiLoginId;
        string returnURL;
        string continueURL;
        string secTranKey;
        string versionNumber;

        //Enter your values here:
        apiLoginId = "EnterYourLoginIdHere";  //Login ID from PaymentsGateway SWP admin
        returnURL = ""; //Optional: must match URL set in PaymentsGateway.net admin settings, page should have script to read http stream
        continueURL = ""; //Optional: used only if returnURL is not set. Page to return to from SWP confirmation screen
        secTranKey = "EnterYourTransKeyHere"; //SecureTransKey in ClearText 
        versionNumber = "1.0"; //SWP version number, per documentation currently 1.0

        //Begin building page stream to post to SWP
        RemotePost myremotepost = new RemotePost();
        myremotepost.Url = "https://swp.paymentsgateway.net/default.aspx";
        myremotepost.Add("pg_api_login_id", apiLoginId);
        myremotepost.Add("pg_return_url", returnURL);
        myremotepost.Add("pg_utc_time", page_pg_utc_time.Value);
        myremotepost.Add("pg_transaction_order_number", page_pg_transaction_order_number.Value);
        myremotepost.Add("pg_continue_url", continueURL);  

        //Generate Hash Value
        string hash;
        hash = generateHash(secTranKey, apiLoginId, page_pg_transaction_type.SelectedValue, versionNumber, page_pg_total_amount.Text, page_pg_utc_time.Value, page_pg_transaction_order_number.Value);
        myremotepost.Add("pg_ts_hash", hash);
        
        //Required Values-- from aspx page fields
            //Note- transaction type options are Credit Card Sale or Check Sale only
            //Other types (Credits & Auths) could be added using a control that stores the appropriate values, as listed in the SWP Integration Guide
            //Valid values as of 2009-02-26: 10=CC Sale, 13=CC Credit, 20=eCheck Sale, 23=eCheck Credit, 11=CC Auth, 21=eChech Auth
        myremotepost.Add("pg_transaction_type", page_pg_transaction_type.SelectedValue);
        myremotepost.Add("pg_version_number", versionNumber);
        myremotepost.Add("pg_total_amount", page_pg_total_amount.Text);

        //Optional Page Values
            //Set any values from page controls you add to the page here to pass to the SWP page.
            //Don't forget to add any required field validators on the page to avoid type errors!
            //See SWP integration guide for for available values
            //Use the following format:
        //myremotepost.Add("pg_[field_name]", yourPageControl.Text);
        
        //Post the stream to the Secure Web Pay processor
        myremotepost.Post();
    }

    private string generateHash(string secTranKey, string apiLoginID, string transactionType, string versionNumber, string totalAmount, string utcTime, string transactionOrderNumber)
    {
        //Build Send String
        string send;
        send = apiLoginID + "|" + transactionType + "|" + versionNumber + "|" + totalAmount + "|" + utcTime + "|" + transactionOrderNumber;

        //Generate and return the hash value
        GenerateHashCode.HashCode objHashkey = new GenerateHashCode.HashCode();
        return objHashkey.GenerateHash(send, secTranKey).ToString();

    }

    public class RemotePost
    {
        //Thanks to Jigar Desai's article at C-Sharp Corner for this class 
        //http://www.c-sharpcorner.com/UploadFile/desaijm/ASP.NetPostURL11282005005516AM/ASP.NetPostURL.aspx
        private System.Collections.Specialized.NameValueCollection Inputs = new System.Collections.Specialized.NameValueCollection();

        public string Url = "";
        public string Method = "post";
        public string FormName = "form1";

        public void Add(string name, string value)
        {
            Inputs.Add(name, value);
        }

        public void Post()
        {
            System.Web.HttpContext.Current.Response.Clear();

            System.Web.HttpContext.Current.Response.Write("<html><head>");

            System.Web.HttpContext.Current.Response.Write(string.Format("</head><body onload=\"document.{0}.submit()\">", FormName));
            System.Web.HttpContext.Current.Response.Write(string.Format("<form name=\"{0}\" method=\"{1}\" action=\"{2}\" >", FormName, Method, Url));
            for (int i = 0; i < Inputs.Keys.Count; i++)
            {
                System.Web.HttpContext.Current.Response.Write(string.Format("<input name=\"{0}\" type=\"hidden\" value=\"{1}\">", Inputs.Keys[i], Inputs[Inputs.Keys[i]]));
            }
            System.Web.HttpContext.Current.Response.Write("</form>");
            System.Web.HttpContext.Current.Response.Write("</body></html>");

            System.Web.HttpContext.Current.Response.End();
        }
    }

    }
